/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */

#include <QAbstractItemModel>
#include <QVector>
#include <QStringList>
#include "organiser_export.h"

class AbstractStringListModelPrivate;

/**
 * \brief This is a minimal string-based model for representing
 * arbitrary MList model.
 *
 * The basic idea is to have one (1) string that represents the whole
 * model, and then 1 string per entry being shown; additionally group
 * headers are included, so the whole input data structure looks as
 * follows:
 *
 * (global-key, qlist<group-header, qlist<local-key>>)
 *
 * Of course, the typical usecase here is that 'key' is uid for the
 * backend, and therefore we can (for example) have incidence uid as
 * global-key, and then attendee uids as local-keys.
 *
 * The ::data call results are QStringList, with global-key and
 * local-key as content.
 *
 * Note: This model implementation is fairly minimal due to the
 * assumption that it does NOT mutate; it is constant, and new
 * instance is created if needed.
 */
class ORGANISER_EXPORT AbstractStringListModel : public QAbstractItemModel
{
    Q_OBJECT
public:

    /**
     * Datastructure used to represent contents of a single
     * groupheader in MList.
     *
     * The first in the QPair is the group header text itself, and the
     * second is a list of strings which are returned on the ::data()
     * calls alongside the global key to the caller.
     */
    typedef QPair<QString,QStringList> SingleGroupEntry;

    /**
     * Datastructure used to represent the whole contents of the list
     * (except for the group key).
     *
     * A vector is used to make the internal initialization more
     * efficient.
     */
    typedef QVector<SingleGroupEntry> GroupEntries;
    
    /**
     * Constructor - Initialize the list with fixed entries.
     *
     * \param gk is the global key that is provided in all ::data() calls.
     * \param d describes the per-groupheader vector described above,
     * which contains groupheader + list of strings each.
     *
     * Any other constructors are intentionally omitted as this is an
     * immutable class, effectively; once initialized, contents cannot
     * be changed.
     */
    AbstractStringListModel(const QString &gk, const GroupEntries &d);

    /**
     * Destructor
     */       
    ~AbstractStringListModel();


    //! \reimp
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;

    QModelIndex index(int, int, const QModelIndex&) const;
    QModelIndex parent(const QModelIndex&) const;
    int columnCount(const QModelIndex&) const;
    //! \reimp_end
    
protected:
    Q_DISABLE_COPY(AbstractStringListModel);
    Q_DECLARE_PRIVATE(AbstractStringListModel);
    AbstractStringListModelPrivate* d_ptr;
};

