/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * This file defines the CalendarConf class which is used for storing
 * configuration variables.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef CALENDARCONF_H
#define CALENDARCONF_H

#include <QObject>
#include "organiser_export.h"

class CalendarConfPrivate;

/**
 * \brief A class for storing configuration variables.
 *
 */
class ORGANISER_EXPORT CalendarConf : public QObject
{
  Q_OBJECT

    static CalendarConf *mSingleton;
public:
  /**
   * \brief Returns a singleton instance of the class.
   */
  static CalendarConf& instance();

  /**
    * \brief Destroy the singleton
    */
  static void destroy();

  /**
   * \brief Get the time zone support setting.
   *
   * \return True if the setting is on, false if off.
   */
  bool timeZoneSupportOn();

  /**
   * \brief Set time zone support on/off.
   *
   * \param value True if the support is on, false if off.
   */
  void setTimeZoneSupportOn(bool value);

  /**
   * \brief Get the first day of week.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \return First day of week.
   */
  int firstDayOfWeek();

  /**
   * \brief Set the first day of week.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \param day First day of week.
   */
  void setFirstDayOfWeek(int day);

  /**
   * \brief Get the working days mask.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \return Bitwise OR'd value of days.
   * \see workingDay
   * favor of bitmasks.
   */
  int workingDaysMask();

  /**
   * \brief Set the working days mask.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \param mask Bitwise OR'd value of days.
   * \see setWorkingDay
   */
  void setWorkingDaysMask(int mask);

  /**
   * \brief Determine if the given day is a working day.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \param day The day.
   * \return True if the day is a working day, false otherwise.
   */
  bool workingDay(int day);

  /**
   * \brief Set the day's working day status.
   *
   * Valid days are 1-7: monday-sunday.
   *
   * \param day The day for which to set the status.
   * \param workingDay If true, the day is set as a working day. If false,
   * the day is set as a non-working day.
   */
  void setWorkingDay(int day, bool workingDay);

  /**
   * \brief Get 'hide completed todos' setting.
   *
   * \return True if completed todos should be hidden, false otherwise.
   */
  bool hideCompletedTodos();

  /**
   * \brief Set 'hide completed todos' setting.
   *
   * \param value True if completed todos should be hidden, false otherwise.
   */
  void setHideCompletedTodos(bool value);

  /**
   * \brief Get 'has default reminder' setting.
   *
   * \return Flag witch shows if default reminder value was set.
   */
  bool hasDefaultReminder();

  /**
   * \brief Set 'has default reminder' setting.
   *
   * \param value true if value was set and false otherwise.
   */
  void setHasDefaultReminder(bool value);

  /**
   * \brief Get 'default reminder' setting.
   *
   * \return Default reminder time in minutes.
   */
  int defaultReminder();

  /**
   * \brief Set 'default reminder' setting.
   *
   * \param value Default reminder time in minutes.
   */
  void setDefaultReminder(int value);

  /**
   * \brief Get 'last active tab' setting.
   *
   * \return Last active tab identifier.
   */
  int lastActiveTab();

  /**
   * \brief Set 'last active tab' setting.
   *
   * \param value Last active tab identifier.
   */
  void setLastActiveTab(int value);

  /**
   * \brief Get count of days for week view.
   *
   * \param value count of days.
   */
  int weekViewDays();

  /**
   * \brief Set count of days for week view.
   *
   * \param value count of days.
   */
  void setWeekViewDays(int value);

  /**
   * \brief Get count of days for week view.
   *
   * \return first run flag value.
   */
  bool isFirstStart() const;

  /**
   * \brief Set first run flag.
   *
   * \param value flag value.
   */
  void setFirstStartFlag(bool value);

  /**
   * \brief Is the notes DB fully empty?
   *
   * \return This value is not 100% right all the time.
   */
  bool notesWithContent() const;

  /**
   * \brief If some note is save into db (journal) this flag should be set to true
   *
   * \param value flag value.
   */
  void setNotesWithContent(bool value);

signals:
  /**
   * \brief Emitted when any of the values has changed.
   */
  void dataUpdated();

private slots:
  /**
   * \brief Called when settings file has changed.
   */
  void settingsFileChanged(const QString & path);

  /**
   * \brief Called when single shot timer expires.
   */
   void syncSettingsCallback();

private:
  Q_DISABLE_COPY(CalendarConf);
  Q_DECLARE_PRIVATE(CalendarConf);
  CalendarConfPrivate* d_ptr;

  /**
   * \brief Constructor.
   */
  CalendarConf();

  /**
   * \brief Destructor.
   */
  virtual ~CalendarConf();

  friend class Ut_CalendarConf;

};

#endif
