/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines DbInstance class.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef DB_INSTANCE_H
#define DB_INSTANCE_H

#include <QObject>
#include <extendedcalendar.h>
#include <incidence.h>
#include <kdatetime.h>
#include <extendedstorage.h>
#include <extendedstorageobserver.h>
#include <timed/interface.h>
#include "organiser_export.h"

class DbInstancePrivate;

/**
 * \brief
 * A class for accessing the database. By default the user is calendar, and
 * the caches are initialized for Calendar. If it is used from Notes it has
 * to be set by setUser();
 */
class ORGANISER_EXPORT DbInstance : public QObject,
                                    public mKCal::ExtendedStorageObserver
{
  Q_OBJECT

public:

  enum DbUser {
    UserCalendar,     // Initial load of calendar events for current month
    UserNotes,        // Initial load of notes
    UserNoLoading     // No initial load of anything
  };

  enum DbSaveError {
    SaveErrorOk,      // Data saved in DB
    SaveErrorFail,    // Data wasn't saved in DB
    SaveErrorIO       // IO error occured during operation
  };

  /**
   * \brief Get a singleton instance of the class.
   */
  static DbInstance& instance();

  /**
   * \brief Set the program that will use the dbInstance
   * The caching is different, in calendar we cached Todos/Events
   * and in notes we cache Journals
   */
  void setUser(DbUser userType);

  /**
   * \brief Return a pointer to calendar object.
   */
  mKCal::ExtendedCalendar::Ptr& calendar();

  /**
   * \brief Return a pointer to storage object.
   */
  mKCal::ExtendedStorage::Ptr& storage();

  /**
   * \brief Return whether storage open succeeded or not.
   */
  bool didDatabaseOpenSucceed();

  /**
   * \brief Retry database open.
   */
  void retryDatabaseOpen();

  /**
   * \brief Return the database file name.
   */
  QString databaseFileName();

  /**
   * \brief Recreate the database instance.
   *
   * This closes and deletes the active storage & calendar objects, freeing
   * any cached data, and recreates them. Use for reducing memory usage.
   * A dataUpdated signal is emitted after the operation is complete.
   *
   */
  void recreate();

  /**
   * \brief Destroy the internal state.
   *
   * This utility call clears up the internal state; this is needed as
   * there may be dependencies for which the default (random)
   * destructor execution order doesn't suffice.
   */
  void destroy();
  
  /**
   * \brief Called when system time or timezone has changed.
   */
  void systemModified();

  /**
   * \brief Alarms need to be recalculated because time is changed
   */
  void recalculateAlarms();
  
  /**
   * \brief Called by storage to notify of modifications.
   */
  void storageModified( mKCal::ExtendedStorage *storage, const QString &info );

  /**
   * \brief Called by storage to notify of modifications.
   */
  void storageProgress( mKCal::ExtendedStorage *storage, const QString &info );

  /**
   * \brief Called by storage to notify of modifications.
   */
  void storageFinished( mKCal::ExtendedStorage *storage, bool error, const QString &info );

  /**
   * \brief Utility function for getting incidence by uid.
   *
   *
   * This function first attempts to retrieve the incidence from the
   * calendar, if it fails, attempts to load it from storage, and
   * finally checks calendar again after the load.
   *
   * \param uid The uid of the incidence to be loaded.
   * \param recurrenceId The particular recurrenceId to be loaded.
   */
  KCalCore::Incidence::Ptr incidence( const QString &uid,  const KDateTime &recurrenceId = KDateTime() );

  /**
   * \brief Inform the backend to load the recurring Incidences
   */
  void loadRecurringIncidences();

  /**
   * \brief Inform the backend to load the Uncompleted Todos
   */
  void loadUncompletedTodos();

  /**
   * \brief Inform the backend to load the Journals
   */
  void loadJournals();

  /**
   * \brief Inform the backend to load a range of dates.
   * This class is aware of the range loaded and tries to avoid
   * going to DB as much as possible.
   */
  void load(const QDate &start, const QDate &end);

  /**
   * \brief Inform the backend to load incidences of a day if they are not loaded
   */
  void load(const QDate &day);

  /**
   * \brief Returns if the database access is enabled or not
   */
  bool isEnabled();

   /**
   * \brief Inform the backend to load future incidences after the current day
   */
  int loadFutureIncidences(int batchSize, KDateTime *day);

  /**
   * \brief Update a notebook checking first if we have a thread doing somehting with the storage
   */
  void updateNotebook(const mKCal::Notebook::Ptr &nb);

  /**
   *
   * Deprecated: This is now done in backend (mkcal/extendedcalendar)
   *
   * \brief Figure when particular recurrence of an incidence starts.
   *
   * The start is a hint that is used as rough approximation (it may
   * be result of expandMultiDay).
   */
  static QDateTime incidenceRecurrenceStart( KCalCore::Incidence::Ptr &incidence, QDateTime start );

  /**
   * Deprecated: This is now done in backend (mkcal/extendedcalendar)
   *
   * \brief Figure appropriate end time for incidence.
   *
   * This functon should be used only on results of expandMultiDay,
   * and therefore the start may or may not be incidence start time.
   *
   * \param start The start time gotten from say, expandMultiDay
   *
   * \param endWithinDay Whether we should try to end within the day or not.
   *
   * \return The time when the event ends. While this function returns
   * QDateTime, the date will be always same as start.date(); this is
   * just convenience API. Only exception is all-day case, where date
   * = start.date()+1, and time is 0,0,0.
   */
  static QDateTime incidenceEndTime( KCalCore::Incidence::Ptr &incidence,
                                     QDateTime start,
                                     bool endWithinDay );


private:
  Q_DISABLE_COPY(DbInstance);
  Q_DECLARE_PRIVATE(DbInstance);

  DbInstancePrivate* d_ptr;

  DbInstance();
  ~DbInstance();

signals:
  /**
   * \brief Emitted when the data in the database has been updated and the
   * users of this class should update their views.
   */
  void dataUpdated();

  void dbEnabled();

  void dbOpenedOk();

  void dbOpenedNok();

  void dbSaved(DbInstance::DbSaveError code);

public slots:
  void timeSettingsChanged(const Maemo::Timed::WallClock::Info &info,
                           bool time_changed) ;

  /**
   * \brief Enable the DB access from the application
   * On startup the db conection is temporary disable, and once it is
   * finished this function should be called to enable the DB access
   * again.
   */
  void enableDb();

  /**
   * \brief Save the pending changes into the permantent storage
   * \param ignoreOwnSave true if we want to force reloading, false otherwise (default is false)
   */
  bool save(bool ignoreOwnSave = false);

  /**
   *  \brief Tries to open the Db.
   * If it can open it inmediately it returs ok, if not, it will
   * try asynchronouse for several times.
   * The proper signals will be emited if finally opened of failed
   * The Db cannot be anabled until the open is succesful
   */
  bool tryOpenDatabase();


  /**
   * \brief Does the initial load in a separate thread which is destroyed afterwards
   */
  void threadedInit();

private slots:

  void updateNotebookAsync();

};

#endif
