/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the MultiStateWidget.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef MULTISTATEWIDGET_H
#define MULTISTATEWIDGET_H

#include <MWidgetController>
#include <QObject>

#include "multistatewidgetmodel.h"
#include "multistatewidgetstyle.h"
#include "organiser_export.h"

class MultiStateWidgetPrivate;

/**
 * \brief A container widget with multiple layouts.
 *
 * This container is mostly useful in situations where a widget contains
 * several subwidgets that have different layouts depending on external
 * factors (such as values entered by the user) and orientation of the device.
 * Changing each subwidget's layout on orientation change (taking the
 * external factors into account) and changing the layouts when external
 * factors change (this time taking the orientation into account) requires
 * a lot of bookkeeping and might prove error-prone.
 *
 * This container simplifies the task by letting the programmer define
 * the different states (e.g. state1-portrait, state1-landscape, state2-...)
 * first and then switch between states with call to changeState(). Orientation
 * change is handled automatically.
 */
class ORGANISER_EXPORT MultiStateWidget : public MWidgetController
{
  Q_OBJECT
  M_CONTROLLER(MultiStateWidget)

public:
  /**
   * \brief Constructor.
   */
  MultiStateWidget();

  /**
   * \brief Constructor.
   */
  MultiStateWidget(QGraphicsItem *parent);

  /**
   * \brief Destructor.
   */
  virtual ~MultiStateWidget();

  /**
   * \brief Add a new state to the widget.
   *
   * @param num Number of the state.
   * @param landscapePolicy The layout policy for this state in landscape mode.
   * @param portraitPolicy The layout policy for this state in portrait mode.
   */
  void addState(int num, MAbstractLayoutPolicy* landscapePolicy, MAbstractLayoutPolicy* portraitPolicy);

  /**
   * \brief Change the widget's state to the given state.
   *
   * @param num Number of the new state.
   */
  void changeState(int num);

private:
  MultiStateWidgetPrivate* d_ptr;

  Q_DECLARE_PRIVATE(MultiStateWidget);
};

#endif
