/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the NotebookManager class.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTEBOOKMANAGER_H
#define NOTEBOOKMANAGER_H

#include <incidence.h>
#include <notebook.h>
#include <QObject>
#include "organiser_export.h"

using namespace mKCal;

class NotebookManagerPrivate;

/**
 * \brief A class for managing (e.g. creating and deleting) notebooks and
 * their properties.
 */
class ORGANISER_EXPORT NotebookManager : public QObject
{
  Q_OBJECT

public:

 /**
  * \brief Error codes of add, update and delete operations.
  */
  enum ModifyError {
    ModifyErrorOk,      // Data updated in DB
    ModifyErrorFail,    // Data wasn't updated in DB
    ModifyErrorIO       // IO error occured during operation
  };

  /**
   * \brief Returns the NotebookManager singleton instance.
   */
  static NotebookManager& instance();

  /**
   * \brief Adds the notebook to the storage.
   *
   * @param notebook The notebook to add.
   * @return True if adding succeeded, false otherwise.
   */
  bool addNotebook(const Notebook::Ptr& notebook);

  /**
   * \brief Updates the notebook to the storage.
   *
   * @param notebook The notebook to update.
   * @return True if update succeeded, false otherwise.
   */
  bool updateNotebook(const Notebook::Ptr& notebook);

  /**
   * \brief Delete the notebook from storage.
   *
   * @param notebook The notebook to delete.
   * @return True if deletion succeeded, false otherwise.
   */
  bool deleteNotebook(const Notebook::Ptr& notebook);

  /**
   * \brief Returns a list of notebooks.
   */
  Notebook::List notebooks() const;

  /**
   * \brief Finds and returns a notebook by its unique ID.
   * 
   * @param Pointer to notebook or 0 if not found.
   */
  Notebook::Ptr getNotebook(const QString& id) const;

  /**
   * \brief Returns the current default notebook.
   */
  Notebook::Ptr getDefaultNotebook() const;

  /**
   * \brief Set the notebook of a given Incidence.
   */
  bool setIncidenceNotebook(KCalCore::Incidence::Ptr incidence,
                            const QString &id);

private:
  NotebookManager();
  ~NotebookManager();

  Q_DISABLE_COPY(NotebookManager);
  Q_DECLARE_PRIVATE(NotebookManager);
  NotebookManagerPrivate* d_ptr;

signals:
  /**
   * \brief Emitted when the notebooks have changed.
   */
  void notebooksChanged();

  /**
   * \brief Emitted as result of add,update and delete operation.
   */
  void notebooksModified(NotebookManager::ModifyError error);

private slots:
  /**
   * \brief Called when the database has changed.
   */
  void dataUpdated();

};

#endif
