/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the NotebookWidget.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTEBOOKWIDGET_H
#define NOTEBOOKWIDGET_H

#include <MWidgetController>
#include <notebook.h>
#include <QObject>

#include "notebookmanager.h"
#include "notebookwidgetmodel.h"
#include "notebookwidgetstyle.h"
#include "organiser_export.h"

class NotebookWidgetPrivate;

/**
 * \brief
 * A widget that displays a NoteBook object's name and color.
 */
class ORGANISER_EXPORT NotebookWidget : public MWidgetController
{
  Q_OBJECT
  M_CONTROLLER(NotebookWidget)

public:
  /**
   * \brief Constructor.
   *
   * \param showActions Whether to show object menu actions for the widget.
   */
  NotebookWidget(bool showActions = false);

  /**
   * \brief Constructor.
   *
   * \param parent Parent item.
   * \param showActions Whether to show object menu actions for the widget.
   *
   * \todo Remove default c'tor and set parent = 0 on next ABI change..
   */
  NotebookWidget(QGraphicsItem *parent, bool showActions = false);

  /**
   * \brief Set whether object menu actions are shown.
   *
   * \param showActions Whether to show object menu actions for the widget.
   */
  void setShowActions(bool showActions);

  /**
   * \brief Set the notebook to be displayed.
   */
  void setNotebook(const mKCal::Notebook& notebook);

  /**
   * \brief Set the notebook to be displayed by ID.
   */
  void setNotebook(const QString& id);

  /**
   * \brief Set the default notebook as the notebook to be displayed.
   */
  void setNotebookToDefault();
  
  /**
   * \brief Return a list of notebook names.
   */
  QStringList notebookNames() const;

  void notebookClicked();

  /**
   * \brief Destructor.
   */
  virtual ~NotebookWidget();

protected:
  //! \reimp
  void mousePressEvent(QGraphicsSceneMouseEvent* event);
  void mouseMoveEvent(QGraphicsSceneMouseEvent* event);
  void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
  void cancelEvent(MCancelEvent *event);
  void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  //! \reimp_end

private:
  NotebookWidgetPrivate* d_ptr;

  /**
   * \brief Reset the model using the notebook given.
   */
  void resetModel(const QString& id);

  /**
   * \brief Reset the model using the notebook given.
   */
  void resetModel(const mKCal::Notebook& notebook);

  Q_DECLARE_PRIVATE(NotebookWidget);

signals:

  /**
   * \brief The active notebook's id changed.
   */
  void notebookIdChanged(const QString &name);

  /**
   * \brief Sent if a notebook was selected by user (that is, the
   * clickable text portion was clicked by the user).
   */
  void notebookSelected(const QString &id);

  /**
   * \brief Sent when the notebook wants to become in/visible in reaction to
   * an object menu command.
   */
  void requestVisibilityChange(QString notebookId, bool visible);

  /**
   * \brief Sent if a notebook delete option was selected  by user.
   */
  void notebookDeleted(const QString &notebookId);

protected slots:
  //! \reimp
  virtual void updateData(const QList<const char*>& modifications);
  //! \reimp_end
  void notebooksChanged();
  void notebookVisible(bool visible);
  void visibleActionTriggered();
  void invisibleActionTriggered();
  void deleteActionTriggered();
};


/**
 * Convenience class that fires up the NotebookWidgetPickerView based
 * NotebookWidget.
 */
class ORGANISER_EXPORT NotebookWidgetPicker : public NotebookWidget
{
public:
    NotebookWidgetPicker()
    {
        setViewType("picker");
    }

    NotebookWidgetPicker(QGraphicsItem *parent, bool showActions = false)
        : NotebookWidget(parent, showActions)
    {
        setViewType("picker");
    }

    void setShownType(NotebookWidgetModel::NoteType type)
    {
        model()->setShownType(type);
    }
};

/**
 * Convenience class that fires up the NotebookWidgetSettingsView based
 * NotebookWidget.
 */
class ORGANISER_EXPORT NotebookWidgetSettings : public NotebookWidget
{
public:
    NotebookWidgetSettings()
    {
        setViewType("settings");
    }

    NotebookWidgetSettings(QGraphicsItem *parent, bool showActions = false)
        : NotebookWidget(parent, showActions)
    {
        setViewType("settings");
    }
};

class ORGANISER_EXPORT NotebookWidgetColor : public NotebookWidget
{
public:
    NotebookWidgetColor()
    {
        setViewType("color");
    }

    NotebookWidgetColor(QGraphicsItem *parent, bool showActions = false)
        : NotebookWidget(parent, showActions)
    {
        setViewType("color");
    }
};

#endif
