/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the controller for NoteEditWidget.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTEEDITWIDGET_H
#define NOTEEDITWIDGET_H

#include <kdatetime.h>

#include "noteeditwidgetmodel.h"
#include "organiserutility.h"
#include "organiser_export.h"

class NoteEditWidgetPrivate;

/**
* \brief A widget for editing notes.
*/
class ORGANISER_EXPORT NoteEditWidget : public MWidgetController,
                       public VisibilityNotificationIF
{
  Q_OBJECT
  M_CONTROLLER(NoteEditWidget)

public:
  /**
   *
   */
  enum EditMode
  {
      EditModeFull,
      EditModeInvitations
  };

  enum LoadMode
  {
      LoadModeAll,
      LoadModeSingle
  };

  /**
   * \brief Constructor.
   */
  NoteEditWidget();

  /**
   * \brief Destructor.
   */
  ~NoteEditWidget();

  /**
   * \brief Creates a new note for editing.
   *
   * \param hasAlarm If true, enable alarm by default.
   */
  void newNote(bool hasAlarm=false, NoteEditWidgetModel::NoteType noteType = NoteEditWidgetModel::Unset);

  /**
   * \brief Load a note for editing.
   *
   * @param id Id of the note to load.
   * @param recurrenceId Recurrence id of the note to load. Default value is
   * no recurrence id.
   * @param mode If mode LoadModeSingle only one occurrennce will be loaded
   * otherwise the whole siquence. Default LoadModeAll
   * @param start This value is used only when LoadModeSingle to identify occurence.
   * @return True if the load was successful, false otherwise.
   */
  bool loadNote(const QString& id, const KDateTime& recurrenceId = KDateTime(), LoadMode mode = LoadModeAll, const KDateTime& start = KDateTime());

  /**
   * \brief Returns the id of the currently loaded note.
   */
  const QString& noteId() const;

  /**
   * \brief Returns the recurrence id of the currently loaded note.
   */
  const KDateTime& recurrenceId() const;

  /**
   * \brief Get the title.
   *
   * \return Title of the note.
   */
  QString title() const;

  /**
   * \brief Set the title.
   *
   * \param title New title for the note.
   */
  void setTitle(const QString& title);

  /**
   * \brief Get the description.
   *
   * \return Description of the note.
   */
  QString description() const;

  /**
   * \brief Set the description.
   *
   * \param description New description for the note.
   */
  void setDescription(const QString& description);

  /**
   * \brief Get Invitation Status
   */
  KCalStatus inviStatus();

  /**
   * \brief Set Invitation Status
   *
   * \param New InvitationStatus
   */
  void setInviStatus(KCalStatus);

  /**
   * \brief Get the start time.
   */
  const QDateTime& startTime();

  /**
   * \brief Set the start time.
   *
   * \param dt New start time.
   */
  void setStartTime(const QDateTime& dt);

  /**
   * \brief Get the end time.
   */
  const QDateTime& endTime();

  /**
   * \brief Set the end time.
   *
   * \param dt New end time.
   */
  void setEndTime(const QDateTime& dt);

  /**
   * \brief Returns true if none of the note edit fiels have been filled,
   * false otherwise.
   */
  bool isEmpty() const;

  /**
   * \brief Returns true if the note edit fields are in the same value
   * as they were when the note was loaded (or new empty one was
   * created).
   */
  bool isUnchanged() const;

  /**
   * \brief Returns true if the note model fields "recurrence" or
   * "recurrenceEndTime" have been changed.
   */
  bool isRecurrenceChanged() const;

  /**
   * \brief Returns true if the note model fields "hasAlarm" or
   * "alarmOffsetMinutes" have been changed.
   */
  bool isAlarmChanged() const;

  /**
   * \brief Returns true if current note has not been saved,
   * false otherwise
   */
  bool isNewNote() const;

  /**
   * \brief Return true if title is empty,
   * false otherwise
   */
  bool isTitleEmpty();

  /**
   * \brief Return whether a note (Todo) is completed or not.
   * \returns true if status is Incidence::StatusCompleted, false otherwise.
   */
  bool isCompleted();

  /**
   * \brief Check whether a note has participants.
   *
   * \return True if there are participants, false if not.
   */
  bool hasParticipants();

  /**
   * \brief Get NoteEditWidgetModel
   *
   */
  NoteEditWidgetModel* getNoteEditWidgetModel();

  /**
   * \brief Return whether we're loading incidence or not.
   */
  bool isLoading();

  /**
   * \brief Signal view to hide/show unnesesary items.
   */
  void hideExpandableItems(bool hide = true);

  /**
   * \brief Signal view to set focus on text field.
   */
  void setFocusOnText();

  /**
   * \brief Signal clients that note have/do not have something to save.
   */
  void updateSaveAvability(bool force = false, bool canSave = true);

protected:
  /**
   * \brief Internal utility to load fields from the given KCalCore::Incidence.
   */
  void loadFromIncidence(const KCalCore::Incidence::Ptr& note);

  /**
   * \brief Internal utility to save fields to the given KCalCore::Incidence.
   */
  void saveToIncidence(const KCalCore::Incidence::Ptr& note, EditMode mode = EditModeFull, bool forceSaveAll = false);

  //! \reimp
  void retranslateUi();
  //! \reimp_end

private:
  Q_DISABLE_COPY(NoteEditWidget);
  Q_DECLARE_PRIVATE(NoteEditWidget);

  NoteEditWidgetPrivate* d_ptr;


  //! \reimp
  void updateContent(int updateTypeMask);
  int updateTypeMask();
  //! \reimp_end

signals:
  /**
   * \brief Emitted when a save was requested but the entered data is
   * invalid or incomplte.
   */
  void invalidData();

  /**
   * \brief Emitted when a note is loaded.
   */
  void noteLoaded();

  /**
   * \brief Emitted to hide/show unnecessary items on view
   */
  void hideExpandSectionItems(bool hide);

  /**
   * \brief Emitted to set focus on required text field
   */
  void focusOnText();

  /**
   * \brief Signal clients that note have something to save.
   */
  void saveAvailable(bool canSave);

  /**
   * Signal for the case when the note UID of the active incidence is changed
   * @param[in] newUid The updated UID of the event
   * @param[in] recurrenceId The updated recurrence ID
   * @note This signal is emitted when an incidence gets
   *       re-created as the user updates its notebook
   */
  void incidenceUidChanged(const QString &newUid, const KDateTime &recurrenceId);

  /**
   * \brief This is method is called when Locale settings are changed.
   */
  void retranslateUI();

public slots:
  /**
   * \brief Saves the note into the database if user entered data is
   * valid. If the data is not invalid, invalidData() signal is emitted.
   *
   * \returns True if the note was saved or there was nothing to save,
   * false if there was something to save but the data could not be saved
   * because it was not valid.
   */
  bool save(EditMode mode = EditModeFull);

  /** 
   * \brief sends response.
   *
   * Saves the updated event, and sends reply to the invitation using
   * ServiceHandler.
   * */
  void sendInvitation();
  /**
   * Request to notify the corresponsing plugin that the user accepted/declined the invitation
   */
  void responseClaimed();

  /**
   * \brief Toggles the completion status of the Todo note.
   *
   * If status is not Incidence::StatusCompleted, sets status to
   * that. If note already has that status, sets status to
   * Incidence::StatusInProcess.
   */
  void toggleCompleted();

  friend class Ut_NoteEditWidget;

};

#endif
