/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the NoteListWidget class.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTELISTWIDGET_H
#define NOTELISTWIDGET_H

#include <MList>
#include <MLabel>
#include <MSeparator>
#include <QSizePolicy>
#include "notewidget.h"
#include "organiserutility.h"
#include "notewidgetitemmodel.h"
#include "organiser_export.h"

class QString;
class NoteListWidgetPrivate;

/**
 * \brief A widget used for displaying a list of notes to the user.
 */
class ORGANISER_EXPORT NoteListWidget : public MList, public VisibilityNotificationIF
{
  Q_OBJECT

public:

    static int StartDateOffset() { return -14;}
    static int EndDateOffset() { return 28;}

  /**
   * \brief Constructor.
   *
   * \param model Item model to use.
   */
  NoteListWidget(AbstractNoteLoader* loader = 0,
                 bool showGroups = true,
                 bool customGroups = true);

  /**
   * \brief Constructor.
   *
   * \param parent Parent item.
   * \param model Item model to use.
   */
  NoteListWidget(QGraphicsItem *parent,
                 AbstractNoteLoader* loader = 0,
                 bool showGroups = true,
                 bool customGroups = true);

  /**
   * \brief Destructor.
   */
  ~NoteListWidget();

  /**
   * \brief Set the item loader.
   *
   * \param loader The item loader. The item loader provides the items
   * to the list.
   */
  void setItemLoader(AbstractNoteLoader* loader);

  /**
   * \brief Set list item style.
   *
   * \param name - This style will be applyed to all items in list.
   * Type of the style NoteWidgetListItemStyle.
   */
  void setItemStyleName(const QString& name);

  /**
   * \brief Get list item style.
   *
   * \return Style witch is applyed to all items in list.
   * Type of the style NoteWidgetListItemStyle.
   */

  const QString itemStyleName();

  /**
   * \brief Convenience API to return a selection of UIDs.
   */
  QList<QString> getSelectionUidList();
  QHash <QString, QDateTime> getSelectionUidHash();

  int noteCount();

  //! \reimp
  virtual void setGeometry(const QRectF& rect);
  virtual void updateContent(int mask);
  virtual int updateTypeMask();
  //! \reimp_end

  /**
   * \brief Switch date to specified date
   */
  void switchDate(const QDateTime& date);

  /**
   * \brief Get enable list refresh state
   * \return True if enabled and false otherwise.
   *
   */
  bool refreshEnabled();

  /**
   * \brief Disable list refresh
   *
   */
  void setRefreshDisabled();

  /**
   * \brief This method finds viewport of parent widgets.
   * \return Pointer to viewport if it exists otherwise 0
   *
   */
  MPannableViewport* parentViewport();

private:

  void setRefreshEnabled();

  Q_DISABLE_COPY(NoteListWidget);
  Q_DECLARE_PRIVATE(NoteListWidget);
  NoteListWidgetPrivate* d_ptr;

signals:

  /**
   * \brief Emitted when any of the note widgets is clicked.
   *
   * It is currently not possible to know which of the NoteWidgets was clicked.
   *
   * \todo Signal handling with NoteWidgets should be reconsidered at some
   * point. Earlier it was thought impossible to access the widgets created
   * with MAbstractCellCreator. Since this is no longer the case, signal
   * handling with NoteWidgets could now be implemented properly.
   */
  void noteClicked();

  /**
   * \brief Emitted whenever the item model of the list is reloaded.
   */
  void listReloaded();

public slots:
  /**
   * \brief Called when content is updated.
   *
   * Used mainly when NoteListWidget is not the main widget for a page.
   */
  void contentUpdated(int mask);

  /**
   * \brief Convenience API to select all.
   */
  void selectAll();

  /**
   * \brief Convenience API to deselect all.
   */
  void deselectAll();

private slots:
  /**
   * \brief Called when the list stops moving.
   */
  void listStopped();

  /**
   * \brief Called when the list position is changed.
   */
  void viewportPositionChanged(const QPointF & position);

  /**
   * \brief Called when the list starts moving.
   */
  void listStarted();

  /**
   * \brief Called when the pointer released.
   */
  void viewportMouseReleased();

  /**
   * \brief Called to check if list scrolling.
   */
  void ensureListScrolled();
};

/**
 * \brief Note widget controller for Events.
 *
 * Simply sets appropriate view type so that the note widget works
 * for Events. Required as MList will not operate correctly if simply
 * view type is dynamically changed for note widgets.
 */

class EventNoteWidget : public NoteWidget
{
    Q_OBJECT
public:
    EventNoteWidget(NoteListWidget* list) : NoteWidget(KCalCore::Incidence::Ptr(), list)
    {
        setViewType("event-note");
    }
};

/**
 * \brief Note widget controller for Todos.
 *
 * Simply sets appropriate view type so that the note widget works
 * for Todos. Required as MList will not operate correctly if simply
 * view type is dynamically changed for note widgets.
 */

class TodoNoteWidget : public NoteWidget
{
    Q_OBJECT
public:
    TodoNoteWidget(NoteListWidget* list) : NoteWidget(KCalCore::Incidence::Ptr(), list)
    {
        setViewType("todo-note");
    }
};

/**
 * \brief Note widget controller for Journals.
 *
 * Simply sets appropriate view type so that the note widget works
 * for Journals. Required as MList will not operate correctly if simply
 * view type is dynamically changed for note widgets.
 */
class JournalNoteWidget : public NoteWidget
{
    Q_OBJECT
public:
    /**
     * \brief Constructs a new widget.
     */
    JournalNoteWidget(NoteListWidget* list) : NoteWidget(KCalCore::Incidence::Ptr(), list)
    {
        setViewType("journal-note");
    }

};

#endif
