/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */


#ifndef NOTELOADER_H
#define NOTELOADER_H

#include <QString>
#include <QDateTime>
#include <incidence.h>
#include <extendedcalendar.h>
#include <extendedstorage.h>
#include "organiser_export.h"

/**
 * Virtual base class for loader backends
 *
 * The assumption is that the backend 'owns' the loader, and not vice
 * versa; therefore, it is responsibility of the backend to delete
 * loader when it is no longer needed.
 */
class AbstractNoteLoaderBackend {
public:
    /**
     * Add a group if it doesn't already exist (multiple calls do
     * nothing).
     */
    virtual void addGroup(const QString& groupName,
                          int sortKey,
                          const QDateTime& start = QDateTime()) = 0;

    /**
     * Add a single incidence. Keeps count of incidences added (zero
     * pointer incidences not counted).
     *
     * \param groupName The name of the group the incidence should be
     * added to.
     *
     * \param dtStart startTime (if applicable) of the incidence.
     * \param dtEnd endTime (if applicable) of the incidence.
     *
     * \param incidence The incidence (if any). NULL is a special case
     * and it will be shown as 'New' button instead.
     */
    virtual void addIncidence(const QString& groupName,
                         QDateTime* dtStart,
                         QDateTime* dtEnd,
                         const KCalCore::Incidence::Ptr &incidence,
                         bool prepend = false) = 0;
};

/**
 * Virtual base class for loaders
 *
 * The underlying idea is simply three operations:
 * - load() which resets loader state + loads initial data (whatever it is)
 * - loadMoreTop() which loads more data above the currently loaded set
 * - loadMoreBottom() which loads more data below the currently loaded set
 */
class ORGANISER_EXPORT AbstractNoteLoader {
public:
    AbstractNoteLoader();
    virtual ~AbstractNoteLoader();

    /**
     * Reset the current state (if any), and load 'default' notes
     * (whatever those are).
     */
    virtual void load() = 0;

    /**
     * Load more notes above the currently loaded ones, if possible.
     */
    virtual void loadMoreTop() = 0;

    /**
     * Load more notes below the currently loaded ones, if possible.
     */
    virtual void loadMoreBottom() = 0;

    /**
     * Set the note loading backend to be used for adding the notes.
     */
    void setBackend(AbstractNoteLoaderBackend* backend);

    /**
     * Set the batch size to be used with the backend. This is
     * optional call, as there's 'some' reasonable default.
     *
     * \param value The batch size to be used. It has to be positive
     * value, that is, greater than zero.
     */
    void setBatchSize(int value);

    /**
     * Override the dbinstance-provided storage.
     */
    void setCalendar(const mKCal::ExtendedCalendar::Ptr& calendar);

    /**
     * Override the dbinstance-provided storage.
     */
    void setStorage(const mKCal::ExtendedStorage::Ptr& storage);

    /**
     * Add incidence to the backend and keep count.
     */
    void addIncidence(const QString& groupName,
                      QDateTime* dtStart,
                      QDateTime* dtEnd,
                      const KCalCore::Incidence::Ptr &incidence,
                      bool prepend = false);

    /**
     * \brief Return the number of incidences added to the item model.
     *
     * Note that the count does not include zero pointer incidences
     * (also known as "new event" buttons.
     */
    int incidenceCount();

    /**
     * Reset the incidence count.
     */
    void resetIncidenceCount();

protected:
    AbstractNoteLoaderBackend* backend();
    int batchSize();
    mKCal::ExtendedCalendar::Ptr calendar();
    mKCal::ExtendedStorage::Ptr storage();

private:
    class AbstractNoteLoaderPrivate* d_ptr;
    Q_DISABLE_COPY(AbstractNoteLoader);
    Q_DECLARE_PRIVATE(AbstractNoteLoader);

};

#endif /* !NOTELOADER_H */
