/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
#ifndef NOTEWIDGET_H
#define NOTEWIDGET_H

#include <QColor>
#include <QObject>
#include <MWidgetController>
#include <incidence.h>

#include "notewidgetmodel.h"
#include "organiser_export.h"

class NoteWidget;
class NoteListWidget;
class NoteWidgetPrivate;

class ORGANISER_EXPORT NoteWidget : public MWidgetController
{
  Q_OBJECT
  M_CONTROLLER(NoteWidget)

public:
  /**
   * \brief Set the receiving object for clicked signals.
   *
   * \param receiver The receiving object.
   * \param slot Slot of the receiving object.
   */
  static void setClickReceiver(const QObject* receiver, const char* slot);

  /**
   * \brief Set the receiving object for editNote signals.
   *
   * \param receiver The receiving object.
   * \param slot Slot of the receiving object.
   */
  static void setEditNoteReceiver(const QObject* receiver, const char* slot);

  /**
   * \brief Set the receiving object for deleteNote signals.
   *
   * \param receiver The receiving object.
   * \param slot Slot of the receiving object.
   */
  static void setDeleteNoteReceiver(const QObject* receiver, const char* slot, Qt::ConnectionType connectionType = Qt::AutoConnection);

  /**
   * \brief Create a new note widget.
   *
   * \param note Note to display.
   * \param mode Mode in which to display the note contents.
   */
  NoteWidget(const KCalCore::Incidence::Ptr& note = KCalCore::Incidence::Ptr(), NoteListWidget* list = 0);

  /**
   * \brief Destructor.
   */
  virtual ~NoteWidget();

  /**
   * \brief Returns the note displayed by this widget instance.
   */
  const KCalCore::Incidence::Ptr note() const;

  /**
   * \brief Returns the start time displayed by this widget instance.
   */
  const QDateTime &startTime() const;
  
  /**
   * \brief Sets the note displayed by this widget instance.
   *
   * \param note The note to display.
   */
  void setNote(const KCalCore::Incidence::Ptr& note);

  /**
   * \brief Sets the note displayed by this widget instance.
   *
   * \param note The note to display.
   */
  void setNote(const KCalCore::Incidence::Ptr& note, const QDateTime &startTm, const QDateTime &endTm);


  /**
   * \brief Set the start time of the note.
   *
   * This method is provided because it is not possible to determine the
   * start time from note data alone in case of recurring notes.
   *
   * \param time Start time of the note.
   */
  void setStartTime(const QDateTime& time);

  /**
   * \brief Get the start time of the note.
   */
  QDateTime startTime();

  /**
   * \brief Set the end time of the note.
   *
   * This method is provided because it is not possible to determine the
   * end time from note data alone in case of recurring notes.
   *
   * \param time End time of the note.
   */
  void setEndTime(const QDateTime& time);

  /**
   * \brief Get the end time of the note.
   */
  QDateTime endTime();

  /**
   * \brief Set the 'squeezed' status.
   *
   * Note: It is up to the view class to decide what (if anything) to do with
   * this information. Check the appropriate view class's documentation.
   *
   * \param value If true, squeeze the widget.
   */
  void setSqueezed(bool value);

  /**
   * \brief Return the 'squeezed' information.
   */
  bool isSqueezed() const;

  /**
   * \brief Return the color associated with this note.
   */
  QColor color() const;

  /**
   * \brief Return the note's type.
   */
  NoteWidgetModel::NoteType noteType() const;

  /**
   * \brief Return true if the note is an all-day note.
   */
  bool isAllDay() const;

  /**
   * \brief Return whether a note (Todo) is completed or not.
   * \returns true if status is Incidence::StatusCompleted, false otherwise.
   */
  bool isCompleted() const;

  /**
   * \brief Toggles the completion status of the Todo note.
   *
   * If status is not Incidence::StatusCompleted, sets status to
   * that. If note already has that status, sets status to
   * Incidence::StatusInProcess.
   */
  void toggleCompleted();

  /**
   * \brief Sets the completion status of the Todo note.
   *
   * \param status The status.
   */
  void setCompleted(bool status);

  /**
   * \brief Saves the current completion status to database.
   */
  void saveCompleted();

protected:
  //! \reimp
  void retranslateUi();
  void mousePressEvent(QGraphicsSceneMouseEvent* event);
  void mouseMoveEvent(QGraphicsSceneMouseEvent* event);
  void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
  void cancelEvent(MCancelEvent *event);
  void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  QVariant itemChange(GraphicsItemChange change,
                      const QVariant &value);
  //! \reimp_end

private:
  Q_DISABLE_COPY(NoteWidget);
  Q_DECLARE_PRIVATE(NoteWidget);
  NoteWidgetPrivate* d_ptr;

protected slots:
  //! \reimp
  virtual void updateData(const QList<const char*>& modifications);
  //! \reimp_end

public slots:
  void editActionActivated();
  void deleteActionActivated();
  void setCompleted();
  void setUncompleted();
  void viewActionActivated();

signals:
  /**
   * \brief Emitted when the note is clicked.
   *
   * \param uid Uid of the note.
   * \param recurrenceId Recurrence id.
   * \param startTime Start time of this occurrence.
   */
  void noteClicked(QString uid, KDateTime recurrenceId, KDateTime startTime);

  /**
   * \brief Emitted when the note should be opened in editor.
   *
   * \param recurrenceId Recurrence id.
   * \param startTime Start time of this occurrence.
   */
  void editNote(QString uid, KDateTime recurrenceId, KDateTime startTime);

  /**
   * \brief Emitted when the note should be deleted.
   *
   * \param uid Uid of the note.
   * \param recurrenceId Recurrence id.
   * \param startTime Start time of this occurrence.
   * \param prefTitle Preferably used note title.
   */
  void deleteNote(QString uid, KDateTime recurrenceId, KDateTime startTime, QString prefTitle);

  /**
   * \brief This is needed for MListView implementation
   */
  void clicked();

  /**
   * \brief This is method is called when Locale settings are changed.
   */
  void retranslateUI();
};

#endif
