/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines the private part of NoteWidgetItemModel.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTEWIDGETITEMMODEL_P_H
#define NOTEWIDGETITEMMODEL_P_H



/**
 * \brief A helper class for storing incidences in a tree structure.
 *
 * Note: Although the API resembles QObject's API, QObject cannot be used
 * because prepending a child object to the parent QObject's children list
 * is not possible, only appending is possible.
 */
class IncidenceObject
{
public:
  /**
   * \brief Constructor.
   *
   * \param incidence Incidence to associate with this object.
   */
   IncidenceObject(const QDateTime* start=0, const QDateTime* end=0, KCalCore::Incidence::Ptr incidence = KCalCore::Incidence::Ptr())
      : m_incidence_uid(QString()), m_parent(0)
  {
      if (start)
          m_start = *start;

      if (end)
          m_end = *end;

      if (incidence) {
          m_incidence_uid = incidence->uid();
          m_incidence_recurid = incidence->recurrenceId();
      }
  }

  /**
   * \brief Destructor.
   */
  virtual ~IncidenceObject()
  {
      m_parent = 0;
  }

  /**
   * \brief Return this object's parent object.
   */
  IncidenceObject* parent() const
  {
    return m_parent;
  }

  /**
   * \brief Set this object's parent.
   *
   * \param parent The parent.
   * \param prepend If true, prepend this object to the parent's chldren
   * list. If false, append.
   */
  void setParent(IncidenceObject* parent, bool prepend=false)
  {
    m_parent = parent;
    if (prepend)
      parent->m_children.prepend(this);
    else
      parent->m_children.append(this);
  }

  /**
   * \brief Return this object's child objects.
   */
  const QList<IncidenceObject*>& children() const
  {
    return m_children;
  }

  /**
   * \brief Recursively delete all descendants of the subtree.
   */
  void deleteChildren()
  {
    for (int i = 0; i < m_children.size(); ++i) {
      m_children[i]->deleteChildren();
      delete m_children[i];
    }
    m_children.clear();
  }

  /**
   * \brief Return the start time associated with this object, if any.
   */
  QDateTime start()
  {
      return m_start;
  }

  /**
   * \brief Set the start time associated with this object.
   */
  void setStart(const QDateTime& start)
  {
      m_start = start;
  }

  /**
   * \brief Return the end time associated with this object, if any.
   */
  QDateTime end()
  {
      return m_end;
  }

  /**
   * \brief Set the end time associated with this object.
   */
  void setEnd(const QDateTime& end)
  {
      m_end = end;
  }

  /**
   * \brief Return the incidence associated with this object.
   *
   * Note: 0-pointer is used as a placeholder for "No events today" list item.
   */
  KCalCore::Incidence::Ptr incidence() const
  {
      return DbInstance::instance().incidence(m_incidence_uid, m_incidence_recurid);
  }

private:
  QDateTime m_start;
  QDateTime m_end;
  QString m_incidence_uid;
  KDateTime m_incidence_recurid;
  IncidenceObject* m_parent;
  QList<IncidenceObject*> m_children;
};

/**
 * \brief A helper class for storing group headers.
 */
class GroupHeader : public IncidenceObject
{
public:
  /**
   * \brief Constructor.
   *
   * \param label Label of the header.
   * \param sortKey Used for sorting group headers that are created as
   * a result of a call to one of the loading functions.
   */
   GroupHeader(const QString& label, int sortKey, QDateTime start = QDateTime())
    : m_label(label), m_sortKey(sortKey)
  {
      setStart(start);
  }

  virtual ~GroupHeader()
  {
  }

  const QString& label() const
  {
    return m_label;
  }

  int sortKey() const
  {
    return m_sortKey;
  }

private:
  QString m_label;
  int m_sortKey;

};

typedef QMap<GroupHeader*, QList<IncidenceObject*> > RetType;

/**
 * \brief Private part of NoteWidgetItemModel.
 */
class NoteWidgetItemModelPrivate : public AbstractNoteLoaderBackend
{
private:
  NoteWidgetItemModel* q_ptr;
  Q_DISABLE_COPY(NoteWidgetItemModelPrivate);
  Q_DECLARE_PUBLIC(NoteWidgetItemModel);


  void addGroup(const QString& groupName,
                int sortKey,
                const QDateTime& start = QDateTime());

  void addIncidence(const QString& groupName,
                    QDateTime* dtStart,
                    QDateTime* dtEnd,
                    const KCalCore::Incidence::Ptr &incidence,
                    bool prepend = false);

public:
  /**
   * \brief Constructor.
   *
   * @param p Pointer to the public class.
   */
  NoteWidgetItemModelPrivate(NoteWidgetItemModel* p);

  /**
   * \brief Destructor.
   */
  ~NoteWidgetItemModelPrivate();

  /**
   * \brief Clear the internal load state (if any).
   */
  void clearLoad();

  /**
   * \brief Wrapper for call to the AbstractNoteLoader::load() of the
   * loader.
   */
  void load();

  /**
   * \brief Wrapper for call to the AbstractNoteLoader::loadMoreTop() of the
   * loader.
   */
  int loadMoreTop();

  /**
   * \brief Wrapper for call to the AbstractNoteLoader::loadMoreBottom() of the
   * loader.
   */
  int loadMoreBottom();

  /**
   * \brief Utility function which takes the accumulated results of
   * AbstractNoteLoader and actually inserts them to the model.
   */
  void finishLoad(bool prepend=false);

  /**
   * \brief A helper function to put the i incidence in ret under the gh
   * GroupHeader.
   *
   * \param prepend If true, prepend the incidence, otherwise append it.
   */
  void addIncidence(RetType& ret, GroupHeader* gh, IncidenceObject* i, bool prepend=false);


  /**
   * \brief Insert the notes into the model.
   *
   * \param prepend If true, prepend the items, otherwise append them.
   */
  void insertIntoModel(const RetType& notes, bool prepend);

  /**
   * \brief Removes everything from the model.
   *
   * \param whether we want to notify others about the chanegs
   */
  void clearModel(bool notify);

  /**
   * \deprecated
   */
  QList<IncidenceObject*> itemList;

  /**
   * \brief A list of the notes' start times.
   *
   * A note's true start time may differ from the time stored in the Incidence
   * object in case of recurring notes.
   */
  QList<QDateTime> startTimes;

  /**
   * \brief State of the loading.
   */
  int state;

  /**
   * \brief The date this class was created.
   *
   * Used for retaining consistency across calls to load functions.
   */
  QDateTime creationTime;

  /**
   * \brief DateTime of the last loaded note.
   */
  KDateTime lastLoadDateTime;

  /**
   * \brief DateTime of the last header where appropriate.
   */
  QDate lastHeaderDate;

  /**
   * \brief Last group header created.
   */
  GroupHeader* lastGroupHeader;

  /**
   * \brief Indicates whether the 'new' button has been added.
   */
  bool newButtonAdded;

  /**
   * \brief Indicates whether the 'new' button should be added.
   */
  bool addNewButton;

  /**
   * \brief Root of the IncidenceObject tree.
   */
  IncidenceObject root;

  /**
   * Do we want to insert headers at all?
   */
  bool insertHeaders;

  /**
   * The source we use for getting notes.
   */
  AbstractNoteLoader* loader;

  /**
   * The temporary data structure used for gathering 'current'
   * set of entries fromt he loader.
   */
  RetType store;

  /**
   * Number of entries loaded into the 'store'.
   */
  int loadedCount;

  /**
   * Number of valid groups loaded into the 'store'.
   */
  int groupCount;
};

#endif
