/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines an abstract base class for note widget list views.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef NOTEWIDGETLISTITEMVIEW_H
#define NOTEWIDGETLISTITEMVIEW_H

#include "abstractnotewidgetview.h"
#include "notewidget.h"
#include "notewidgetmodel.h"
#include "notewidgetlistitemstyle.h"
#include "organiser_export.h"

class MWidget;
class NoteWidgetListItemViewPrivate;

/**
 * \brief A base class for note widget views for use in event/todo lists.
 *
 * This class defines a view with a title, subtitle, additional content
 * widget (displayed on the left), and alarm icon and color stripe inherited
 * from AbstractNoteWidgetView. The widget resembles MContentItem which
 * unfortunately doesn't allow replacing the icon on the left with a
 * custom widget.
 */
class ORGANISER_EXPORT NoteWidgetListItemView : public AbstractNoteWidgetView
{
  Q_OBJECT
  M_VIEW(NoteWidgetModel, NoteWidgetListItemStyle)

public:
  enum DisplayMode {
    ModeSingleTitle,
    ModeTitleAndSubtitle,
    ModeTitleSubtitleAndText,
    ModeTitleAndText
  };

  /**
   * \brief Constructor.
   *
   * \param controller Pointer to controller class instance.
   */
  NoteWidgetListItemView(NoteWidget* controller);

  /**
   * \brief Destructor.
   */
  virtual ~NoteWidgetListItemView();

  /**
   * \brief Set the display mode.
   *
   * \param mode New display mode.
   */
  void setDisplayMode(DisplayMode mode);

  /**
   * \brief Get the title.
   *
   * \return Current title text.
   */
  const QString title();

  /**
   * \brief Set the title.
   *
   * \param text New title.
   */
  void setTitle(const QString& text);

  /**
   * \brief Get the subtitle.
   *
   * \return Current subtitle text.
   */
  const QString subtitle();

  /**
   * \brief Set the title style name.
   *
   * Note: This style will be valid until next applyStyle call that
   *       is why use css instead of this method.
   * \param name New style name.
   */
  void setTitleStyleName(const QString& name);

  /**
   * \brief Set the subtitle.
   *
   * Note: Subtitle is shown only in TitleAndSubtitle display mode.
   *
   * \param text New subtitle.
   */
  void setSubtitle(const QString& text);

  /**
   * \brief Set the subtitle style name.
   *
   * Note: Subtitle is shown only in TitleAndSubtitle display mode.
   *       This style will be valid until next applyStyle call that
   *       is why use css instead of this method.
   * \param name New style name.
   */
  void setSubtitleStyleName(const QString& name);

  /**
   * \brief Get the sub text.
   *
   * \return Current sub text.
   */
  const QString subText();

  /**
   * \brief Set the sub text.
   *
   * \param text New sub text.
   */
  void setSubText(const QString& text);

  /**
   * \brief Set the subtext style name.
   *
   * Note: This style will be valid until next applyStyle call that
   *       is why use css instead of this method.
   * \param name New style name.
   */
  void setSubTextStyleName(const QString& name);

  /**
   * \brief Set the widget displayed on the left.
   */
  void setDetailsWidget(MWidget* widget);

protected:
  //! \reimp
  virtual void setupModel();
  virtual void applyStyle();
  //! \reimp_end

protected Q_SLOTS:
  //! \reimp
  virtual void updateData(const QList<const char*>& modifications);
  //! \reimp_end

private:
  Q_DECLARE_PRIVATE(NoteWidgetListItemView);
  NoteWidgetListItemViewPrivate* d_ptr;

};

#endif
