/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * \file
 * \brief This file contains functions for accessing a singleton
 * MLocale object.
 *
 * \author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef ORGANISERLOCALE_H
#define ORGANISERLOCALE_H

#include <MLocale>
#include <QObject>
#include "organiser_export.h"

class QDateTime;

/**
 * \brief A class that provides a singleton MLocale object.

 * This class is provided because creating a MLocale is quite expensive
 * with many d-bus calls etc.
 */
class ORGANISER_EXPORT OrganiserLocale : MLocale
{
  Q_OBJECT

public:
  /**
   * \brief Returns a singleton instance of the system locale.
   */
  static MLocale& systemLocale();

  /**
   * \brief Returns curent time format.
   *
   */
  static MLocale::TimeFormat24h currentTimeFormat();

  /**
   * \brief Format only 'time' part of datetime using short time format.
   *
   * This is provided because formatDateTime is an expensive operation and by
   * caching formatted strings we optimize number of calls to formatDateTime.
   */
  static QString formatShortTime(const QDateTime& datetime);

  /**
   * \brief Format only 'time' part of a datetime without am/pm markers.
   */
  static QString formatShortTimeWithoutAmPm(const QDateTime& datetime);

  /**
   * \brief Format only the am/pm part of a datetime.
   *
   * Note: If the current locale doesn't use am/pm markers, an empty string
   * is returned.
   */
  static QString formatOnlyAmPm(const QDateTime& datetime);

  /**
   * \brief Provide a wrapper for systemLocale::formatDateTime
   */
  static QString formatDateTime(const QDateTime &dateTime,
                                DateType dateType = DateLong,
                                TimeType timeType = TimeLong);
  static QString formatDateTime(const QDateTime &dateTime,
                                const char *format);

  /**
   * \brief Return reference to the shared MCalendar
   */
  static MCalendar& sharedCalendar();

private:
  Q_DISABLE_COPY(OrganiserLocale);
  OrganiserLocale() { }
  ~OrganiserLocale() { }

};

#endif
