/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * 
 * \brief This file defines various general 'organiser' level
 * utilities that can be used in both Calendar and Notes application.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 * @author Markus Stenberg \<ext-markus.stenberg@nokia.com\>
 */
#ifndef ORGANISERUTILITY_H
#define ORGANISERUTILITY_H

#include <QObject>
#include <MLabel>
#include <QString>
#include <QGraphicsLayout>
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <MWidget>
#include <MStylableWidget>
#include <MWidgetAction>
#include <MSeparator>
#include <MAbstractCellCreator>
#include <MBasicListItem>


#include "pageswitcher.h"
#include "organiser_export.h"
#include "notewidgetstyle.h"
#include "db_instance.h"

#include "notebookmanager.h"


class MButton;
class MLabel;
class QGraphicsLayoutItem;
class MTextEdit;

using namespace mKCal;


/**
 * \brief Simple widget to provide just a rectangular, colored box.
 *
 * Used by various classes that need to indicate notebook colors.
 */

class ORGANISER_EXPORT MyColorLabel : public MStylableWidget
{
public:
    MyColorLabel()
        : m_color(Qt::color0), m_opacity(1.00)
    {
    }

    MyColorLabel(QGraphicsItem *parent)
        : MStylableWidget(parent), m_color(Qt::color0), m_opacity(1.00)
    {
    }

    QColor color() const
    {
        return m_color;
    }

    void setColor(const QColor& color)
    {
        m_color = color;
        update();
    }

    qreal opacity() const
    {
        return m_opacity;
    }

    void setOpacity(const qreal& opacity)
    {
        m_opacity = opacity;
        update();
    }
private:
    virtual void paint(QPainter* painter,
                       const QStyleOptionGraphicsItem* option,
                       QWidget* widget = 0);

private:
    QColor m_color;
    qreal m_opacity;
    M_STYLABLE_WIDGET(NoteWidgetStyle)
};

/**
 * \brief Interface to represent visibility- and database-interested class.
 *
 * The updateContent() is called if and only if an update signal is
 * received, and the containerwidget either becomes visible, or is already
 * visible.
 */
 
class ORGANISER_EXPORT VisibilityNotificationIF
{
public:
  /**
   * \brief Possible update types.
   */
  enum UpdateTypeFlags {
    UpdateNone = 0x0,
    UpdateDb = 0x1,       // Receive database update signals
    UpdateConfig = 0x2,   // Receive config update signals
    UpdateDate = 0x4,     // Receive date changed signals
    UpdateHour = 0x8,     // Receive hour changed signals
    UpdateMinute = 0x10,  // Receive minute signals
    UpdateNotebooks = 0x20// Receive notebookmanager notebooksChanged signals
  };

  /**
   * \brief This pure virtual method is called when the widget needs to update
   * its content.
   *
   * This method is called when:
   *   1) The widget is visible and an update signal is received, or
   *   2) The widget becomes visible and an update signal was received
   *      while it was invisible.
   *
   * \param mask A bitmask of update types.
   */
  virtual void updateContent(int updateTypeMask) = 0;

  /**
   * This method is called to determine which update signals the class
   * should listen to.
   *
   * \return A bitmask of update types.
   */
  virtual int updateTypeMask() = 0;

};

/**
 * \brief Wrap one MWidget (which hopefully implements the
 * VisibilityNotificationIF) inside another, returning the containing widget.
 *
 * The container will call the VisibilityNotificationIF call if and
 * only if applicable according to the notes above.
 */
ORGANISER_EXPORT MWidget *
organiserUtilityWrapVisibilityNotificationIF(VisibilityNotificationIF *child,
                                             MWidget *parent=NULL);

/**
 * \brief Convenience wrapper which allows for use of the
 * VisibilityNotificationIF within constructors.
 *
 * This provides for convenience implementation which performs the
 * organiserUtilityWrapVisibilityNotificationIF on the given widget
 * type, and pretends to just contain that one. widget() accessor is
 * provided for access to the child class.
 */

// TODO: Currently we wind up with extra nesting hierarchy due to
// this; however, it seems preferrable to hide the implementation of
// this 'visibility' logic from it's users..
// (VisibilityContainerWidget -> VisibilityContainerWidgetPrivate -> T)

template <class T>
class ORGANISER_EXPORT VisibilityContainerWidget : public MWidget,
                                  public PageSwitchObserverIface
{
public:
    VisibilityContainerWidget(MWidget* parent) : MWidget(parent)
    {
        child = new T();
        visibility =
          organiserUtilityWrapVisibilityNotificationIF(child, this);
        QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
        layout->setContentsMargins(0, 0, 0, 0);
        setLayout(layout);
        layout->addItem(visibility);

        // This update should be called when the whole hierarhy of
        // QGraphics items is build. We need that because we use
        // this hierarhy in NoteListWidget to find viewport and change its
        // state. This place is the place where it is built from PannableView of
        // LazyLoadPage to NoteListWidget. May be we should update all cantent
        // but for now it is updated only with VisibilityNotificationIF::UpdateDb flag.
        if(child) {
            VisibilityNotificationIF* item = static_cast<VisibilityNotificationIF*>(child);
            item->updateContent(VisibilityNotificationIF::UpdateDb);
        }
    }
    ~VisibilityContainerWidget()
    {
    }
    T* widget()
    {
        return child;
    }
    bool backClicked()
    {
        PageSwitchObserverIface *iface =
            dynamic_cast<PageSwitchObserverIface *>(widget());
        if (iface)
            return iface->backClicked();
        return true;
    }
    
protected:
    T* child;
    MWidget *visibility;
};

/**
 * \brief List cell creator to provide simple list item with single line text.
 *
 */
class ORGANISER_EXPORT OrganiserCommonListItemCreator : public MAbstractCellCreator<MBasicListItem>
{
public:

    OrganiserCommonListItemCreator() {
        // This a little overhead but it is done
        // because of meegotouch bug cell size is
        // determined without taking into count
        // style of the object but using default
        // style of object
        MWidget *cell = new MBasicListItem();
        cell->setObjectName("PopupListItem"); // Set the same style to
        setCellObjectName("PopupListItem");   // avoid problems with height
        size = cell->effectiveSizeHint(Qt::PreferredSize);
        delete cell;
    }

    virtual QSizeF cellSize() const {
        return size;
    }

    virtual void updateCell(const QModelIndex& index, MWidget* cell) const
    {
        MBasicListItem* listItem = qobject_cast<MBasicListItem*>(cell);
        if(!listItem)
            return;

        QVariant data = index.data(Qt::DisplayRole);
        QString label = data.value<QString>();
        listItem->setTitle(label);
    }

private:

    QSizeF size;
};

/**
 * \brief Calendar List item with color label.
 *
  */
class NotebookWidgetItem: public MBasicListItem
{
    Q_OBJECT
    Q_PROPERTY(QColor color READ color WRITE setColor)

public:

    NotebookWidgetItem(): MBasicListItem(),
                          customLayout(0),
                          colorLabel(0)
    {
    }

    QColor color()
    {
        return colorLabelWidget()->color();
    }

    void setColor(const QColor &color)
    {
        colorLabelWidget()->setColor(color);
    }

protected:

    virtual QGraphicsLayout *createLayout()
    {
        if (!customLayout) {
            customLayout = new QGraphicsGridLayout(this);
            customLayout->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
            customLayout->setContentsMargins(0, 0, 0, 0);
            customLayout->setSpacing(0);

            titleLabelWidget()->setObjectName("CommonSingleTitle");
            customLayout->addItem(titleLabelWidget(), 0, 0);
            colorLabelWidget()->setObjectName("NotebookWidgetColorBox");
            customLayout->addItem(colorLabelWidget(), 0, 1, Qt::AlignVCenter);
        }

        return customLayout;
    }

    virtual void clearLayout()
    {
        Q_ASSERT(customLayout);

        for (int i = customLayout->count() - 1; i >= 0; i--) {
            customLayout->removeAt(i);
        }
    }

    MyColorLabel *colorLabelWidget()
    {
        if (!colorLabel) {
            colorLabel = new MyColorLabel(this);
        }

        return colorLabel;
    }

private:

    QGraphicsGridLayout *customLayout;

    MyColorLabel *colorLabel;

};

/**
 * \brief List cell creator for NotebookWidgetItem.
 *
 */
class NotebookWidgetItemCreator : public MAbstractCellCreator<NotebookWidgetItem>
{
public:

    NotebookWidgetItemCreator() {
        // This a little overhead but it is done
        // because of meegotouch bug cell size is
        // determined without taking into count
        // style of the object but using default
        // style of object
        MWidget *cell = new NotebookWidgetItem();
        cell->setObjectName("PopupListItem"); // Set the same style to
        setCellObjectName("PopupListItem");   // avoid problems with height
        size = cell->effectiveSizeHint(Qt::PreferredSize);
        delete cell;
    }

    virtual QSizeF cellSize() const {
        return size;
    }

    virtual void updateCell(const QModelIndex& index, MWidget* cell) const
    {
        NotebookWidgetItem* listItem = qobject_cast<NotebookWidgetItem*>(cell);
        if(!listItem)
            return;

        QVariant data = index.data(Qt::DisplayRole);
        QString id = data.value<QString>();
        Notebook::Ptr nb = NotebookManager::instance().getNotebook(id);
        if ( nb ) {
            listItem->setTitle(nb->name());
            listItem->setColor(nb->color());
        }
    }

private:

    QSizeF size;
};


/**
 * \brief Ask user a yes/no question.
 *
 * The return value is true if and only if user answered 'yes';
 * user terminating the dialog and user selecting 'no' return false.
 *
 * NOTE: MDialog has issues with exec(). You should use the non-blocking
 * version of this function whenever possible.
 */
ORGANISER_EXPORT bool organiserUtilityAskYesNo(QString prompt);



/**
 * \brief Ask the user a yes/no question.
 *
 * The call will return immediately and the user's response will be delivered
 * later to the specified receiver object.
 *
 * \param prompt The prompt to show.
 * \param receiver Receiver object for the 'finished' signal from MDialog.
 * \param slot Receiver slot for the 'finished' signal from MDialog. The response
 * is passed as an int parameter.
 */
ORGANISER_EXPORT void organiserUtilityAskYesNo(const QString& prompt, QObject* receiver, const char* slot);


/**
 * \brief Ask the user a yes/no question.
 *
 * The call will return immediately and the user's response will be delivered
 * later to the specified receiver object.
 *
 * \param title The dialog title.
 * \param prompt The prompt to show.
 * \param receiver Receiver object for the 'finished' signal from MDialog.
 * \param slot Receiver slot for the 'finished' signal from MDialog. The response
 * is passed as an int parameter.
 */

ORGANISER_EXPORT void organiserUtilityAskYesNoWithTitle(const QString& title, const QString& prompt, QObject* receiver, const char* slot);


/**
 * \brief Ask user a yes/no question with title.
 *
 * The return value is true if and only if user answered 'yes';
 * user terminating the dialog and user selecting 'no' return false.
 *
 * NOTE: MDialog has issues with exec(). You should use the non-blocking
 * version of this function whenever possible.
 */
ORGANISER_EXPORT bool organiserUtilityAskYesNoWithTitle(const QString& title, const QString& prompt);

/**
 * \brief Show a banner with an information message to the user.
 *
 * Common way for the app to show banners.
 */
ORGANISER_EXPORT void organiserUtilityShowInfoBanner(QGraphicsScene* scene, QString prompt);

/**
 * \brief Calendar qSort help mehtod
 *
 * Compare visibility state as first priority and creation date as second
 */

 bool notebookLessThan(const mKCal::Notebook::Ptr &p1, const mKCal::Notebook::Ptr &p2);

/**
 * \brief A convenience method that creates a common spacer widget.
 */
MStylableWidget* organiserUtilityCreateCommonSpacer();

/**
 * \brief A convenience method that creates a common spacer widget.
 *
 * \param parent Parent item
 */
MStylableWidget* organiserUtilityCreateCommonSpacer(QGraphicsItem *parent);

/**
 * \brief A convenience method that creates a spacer widget.
 *
 * \param styleName Spacer stylename
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreateSpacer(const QString& styleName);

/**
 * \brief A convenience method that creates a spacer widget.
 *
 * \param styleName Spacer stylename
 * \param parent Parent item
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreateSpacer(const QString& styleName, QGraphicsItem *parent);

/**
 * \brief A convenience method that creates a separator widget.
 *
 * \param styleName Spacer stylename
 */
ORGANISER_EXPORT MSeparator* organiserUtilityCreateSeparator(const QString& styleName, Qt::Orientation orientation = Qt::Horizontal);

/**
 * \brief A convenience method that creates a separator widget.
 *
 * \param styleName Spacer stylename
 * \param parent Parent item
 * \param orientation Separator orientation.
 */
ORGANISER_EXPORT MSeparator* organiserUtilityCreateSeparator(QGraphicsItem *parent,
                                                             const QString& styleName,
                                                             Qt::Orientation orientation = Qt::Horizontal);

/**
 * \brief A convenience method that creates a common panel widget.
 *
 * \param item Item to be wrapped in the panel. Optional.
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreateCommonPanel(QGraphicsLayoutItem* item = 0);

/**
 * \brief A convenience method that creates a common panel widget.
 *
 * \param parent Parent item
 * \param[out] ll The pointer to be set to the new QGraphicsLinearLayout.
 * \param orientation Layout orientation.
 *
     Example:
       QGraphicsLinearLayout* ll;
       MStylableWidget* w =  organiserUtilityCreateCommonPanel(parent, ll);
       ll->addItem(item);
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreateCommonPanel(QGraphicsItem *parent,
                                                                    QGraphicsLinearLayout *&ll,
                                                                    Qt::Orientation orientation = Qt::Horizontal);

/**
 * \brief A convenience method that creates a stylable panel widget.
 *  If item is not 0 it creates layout, set it to panel widget and adds item to layout
 *
 * \param styleName Panel stylename
 * \param item Item to be wrapped in the panel. Optional.
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreatePanel(const QString &styleName, QGraphicsLayoutItem* item=0);

/**
 * \brief A convenience method that creates a stylable panel widget.
 *
 * \param parent Parent item
 * \param styleName Panel stylename
 * \param[out] ll The pointer to be set to the new QGraphicsLinearLayout.
 * \param orientation Layout orientation.
 *
     Example:
       QGraphicsLinearLayout* ll;
       MStylableWidget* w =  organiserUtilityCreatePanel(parent, "CommonPanel", ll);
       ll->addItem(item);
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreatePanel(QGraphicsItem *parent,
                                                              const QString &styleName,
                                                              QGraphicsLinearLayout *&ll,
                                                              Qt::Orientation orientation = Qt::Horizontal);

/**
 * \brief A convenience method that creates a stylable panel widget.
 *
 * \param parent Parent item
 * \param styleName Panel stylename
 */
ORGANISER_EXPORT MStylableWidget* organiserUtilityCreatePanel(QGraphicsItem *parent, const QString &styleName);

ORGANISER_EXPORT MListItem* organiserUtilityCreateListItem(QGraphicsItem *parent, const QString &styleName);

ORGANISER_EXPORT MListItem* organiserUtilityCreateListItem(QGraphicsItem *parent, const QString &styleName, QGraphicsLinearLayout *&ll, Qt::Orientation orientation = Qt::Horizontal);

/**
 * \brief A convenience method that creates a new vertical
 * QGraphicsLinearLayout and sets it's spacing+contentmargin to zero.
 *
 * \param[out] ll The pointer to be set to the new QGraphicsLinearLayout.
 */
ORGANISER_EXPORT void organiserUtilityNewVerticalLayout(QGraphicsLinearLayout *&ll);

/**
 * \brief A convenience method that creates a new vertical
 * QGraphicsLinearLayout and sets it's spacing+contentmargin to zero.
 *
 * \param[out] ll The pointer to be set to the new QGraphicsLinearLayout.
 * \param parent Parent item
 */
ORGANISER_EXPORT void organiserUtilityNewVerticalLayout(QGraphicsLinearLayout *&ll, QGraphicsLayoutItem *parent);

/**
 * \brief A convenience method that creates a new
 * QGraphicsGridLayout and sets it's spacing+contentmargin to zero.
 *
 * \param[out] gl The pointer to be set to the new QGraphicsGridLayout.
 * \param parent Parent item
 */
ORGANISER_EXPORT void organiserUtilityNewGridLayout(QGraphicsGridLayout *&gl, QGraphicsLayoutItem *parent = 0);

/**
 * \brief Convert tracker URI/UID/something to UID.
 *
 * \param s Input string to be parsed.
 *
 * \return UID, or best guess at UID.
 */
ORGANISER_EXPORT QString organiserUtilityStringToUID(const QString &s);

/**
 * \brief Try to get the RecurrenceId from a Tracker URI
 *
 * \param s Input string to be parsed.
 *
 * \return The Date
 */
ORGANISER_EXPORT KDateTime organiserUtilityUriToRecurrenceId(const QString &s);

/**
 * \brief Definition of UID for default personal calendar.
 *
 */
static const QString default_personal_calendar_uid = QString("11111111-2222-3333-4444-555555555555");

/**
 * \brief A convenience method that resets layout margings and spacing
 *
 * \param layout The pointer to layout for reset.
 */
template<class T>
void organiserUtilityResetLayoutMargingsAndSpacing(T* layout)
{
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
}

#endif /* !ORGANISERUTILITY_H */
