/* * This file is part of Harmattan Organiser GUI *
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 * All rights reserved.
 *
 * Contact: Andrey Moiseenko <andrey.moiseenko@nokia.com>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */
/**
 * @file
 * \brief This file defines class(es) for switching between application pages.
 *
 * @author Mikko Herranen \<ext-mikko.herranen@nokia.com\>
 */
#ifndef PAGESWITCHER_H
#define PAGESWITCHER_H

#include <QObject>
#include <MApplicationPage>
#include <MSheet>
#include <MBasicSheetHeader>
#include <MDialog>
#include <QGraphicsLinearLayout>
#include <MPannableViewport>
#include <MPositionIndicator>
#include <MPhysics2DPanning>
#include <MLabel>

#include "organiser_export.h"
#include "pageheaderwidget.h"

class PageSwitcherPrivate;

class ORGANISER_EXPORT OrgMPhysics2DPanning : public MPhysics2DPanning
{
    Q_OBJECT;
public:
    OrgMPhysics2DPanning(QObject *parent);

Q_SIGNALS:
    void pointerReleased();

protected:
    //! \reimp
    void pointerRelease();
};


/**
 * \brief A simple interface class for LazyLoadPage.
 */
class ORGANISER_EXPORT LazyLoadPageIFace : public MApplicationPage
{
public:
  virtual void destroyContent() = 0;
};


class ORGANISER_EXPORT LazyLoadPageHelperIFace
{
public:

    virtual void updateViewportSize() = 0;
};

class ORGANISER_EXPORT LazyLoadPageHelper : public QObject
{
    Q_OBJECT

public:

    LazyLoadPageHelper(LazyLoadPageHelperIFace* page) : m_page(page)
    {
    }

public slots:

    void updateViewportSize()
    {
        if (m_page)
            m_page->updateViewportSize();
    }

private:

    LazyLoadPageHelperIFace* m_page;
};


/**
 * \brief A templated page class that creates a widget of type T
 * and sets it as the central widget when the page is made visible.
 *
 * \note This page class contains two pannable port views:
 *     1) MApplicationPage's view port it must be disabled ever.
 *     2) m_viewport this is active view.
 * That is why don't use MApplication methods setPannable and
 * isPannable. Instead use methods of this class to change panning state.
 * Default state for panning "enabled".
 */
template <class T> class LazyLoadPage : public LazyLoadPageIFace,
                                        public LazyLoadPageHelperIFace
{
public:

  /**
   * \brief Constructor.
   */
  LazyLoadPage()
    :m_widget(0),
    m_header_widget(0),
    m_header_label(0),
    m_helper(new LazyLoadPageHelper(this))
  {
      // We disable MApplication pannable view
      // because otherwise there will be conflict
      // between our pannable view and MApplication's
      setPannable(false);
  }

  /**
   * \brief Destructor.
   */
  ~LazyLoadPage()
  {
      delete m_header_widget;
      m_header_widget = 0;

      delete m_helper;
      m_helper = 0;
  }

  /**
   * \brief Return a handle to the widget.
   *
   * \note Should only be called after page content has been created.
   */
  T* widget()
  {
    return m_widget;
  }

  /**
   * \brief Return a handle to the page header widget.
   *
   * \note Should only be called after page content has been created.
   */
  MStylableWidget *headerwidget()
  {
      Q_ASSERT(m_header_widget);
      return m_header_widget;
  }

  /**
   * \brief Set visibility for the page header widget.
   *
   * \note Should only be called after page content has been created.
   */
  void setHeaderVisible(bool visible)
  {
      if(!m_header_widget)
          return;

    if(visible) {
        if(m_layout->itemAt(0) != m_header_widget) {
            m_layout->insertItem(0, m_header_widget);
            updateViewportSize();
        }
    } else {
        if(m_layout->itemAt(0) == m_header_widget) {
            m_layout->removeItem(m_header_widget);
            updateViewportSize();
        }
    }
  }

  void setHeaderTitle(const QString& title)
  {
      if(!m_header_widget || !m_header_label)
          return;

      m_header_label->setText(title);
  }

  void setHeaderTitleStyle(const QString& style)
  {
      if(!m_header_widget || !m_header_label)
          return;

      m_header_label->setStyleName(style);
  }

  /**
   * \brief Set if view is pannable or not.
   *
   * \note Should only be called after page content has been created.
   * Please use this method to change pannable state of the contents
   * and don't use methods of MApplication it is error-prone.
   */
  void setViewPannable(bool pannable)
  {
      if(!m_viewport)
          return;

      if(pannable) {
          m_viewport->setPanDirection(Qt::Vertical);
      } else {
          m_viewport->setVerticalPanningPolicy(MPannableWidget::PanningAlwaysOff);
          m_viewport->setHorizontalPanningPolicy(MPannableWidget::PanningAlwaysOff);
      }
  }

  /**
   * \brief Get page viewport
   *
   * \note Should only  be called after page content has been created.
   */
  MPannableViewport* viewPort() const
  {
      return m_viewport;
  }

  /**
   * \brief Called by the application framework to create page content before
   * displaying the page.
   *
   * \note The application framework will call this function only once.
   * If you use destroyContent() to destroy the page contents, you must
   * call createContent() explicitly before displaying the page.
   */
  virtual void createContent()
  {
    if (m_widget)
      return;

    QObject::connect(this, SIGNAL(exposedContentRectChanged()), m_helper, SLOT(updateViewportSize()));

    m_layout = new QGraphicsLinearLayout(Qt::Vertical);
    m_layout->setContentsMargins(0, 0, 0, 0);
    m_layout->setSpacing(0.0);    

    MWidget *w = new MWidget();
    m_viewport = new MPannableViewport(w);
    m_viewport->setPreferredHeight(pannableViewport()->preferredHeight());
    m_viewport->setMinimumHeight(pannableViewport()->minimumHeight());
    OrgMPhysics2DPanning* physics2D = new OrgMPhysics2DPanning(m_viewport);
    m_viewport->setPhysics(physics2D);
    m_viewport->setClipping(true);
    m_viewport->positionIndicator()->setEnabled(false);
    setViewPannable(true);

    m_widget = new T(m_viewport);

    m_header_widget = new MStylableWidget();
    m_header_widget->setStyleName("CommonHeaderPanel");
    m_header_widget->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));
    QGraphicsLinearLayout* headerLayout = new QGraphicsLinearLayout(Qt::Horizontal, m_header_widget);
    headerLayout->setSpacing(0);
    headerLayout->setContentsMargins(0, 0, 0, 0);
    m_header_widget->setLayout(headerLayout);

    m_header_label = new MLabel(m_header_widget);
    m_header_label->setStyleName("CommonHeader");
    m_header_label->setTextElide(true);

    headerLayout->addItem(m_header_label);

    m_viewport->setWidget(m_widget);

    m_layout->addItem(m_viewport);
    w->setLayout(m_layout);
    setCentralWidget(w);
  }

  /**
   * \brief Destroys the content.
   */
  virtual void destroyContent()
  {
    if (!m_widget)
      return;

    // The widget must be deleted by calling setCentralWidget or we crash
    // when setCentralWidget is called on content recreation.
    setCentralWidget(0);
    m_widget = 0;
  }

  virtual void updateViewportSize()
  {
      if (m_viewport) {
          if (m_layout->itemAt(0) == m_header_widget) {
              m_viewport->setPreferredHeight(exposedContentRect().height()-m_header_widget->size().height());
              m_viewport->setMinimumHeight(exposedContentRect().height()-m_header_widget->size().height());
          } else {
              m_viewport->setPreferredHeight(exposedContentRect().height());
              m_viewport->setMinimumHeight(exposedContentRect().height());
          }
      }
  }

private:
  T* m_widget;
  QGraphicsLinearLayout* m_layout;
  MStylableWidget* m_header_widget;
  MLabel* m_header_label;
  MPannableViewport* m_viewport;
  LazyLoadPageHelper* m_helper;
};

/**
 * \brief A simple interface class for LazyLoadSheet.
 */
class ORGANISER_EXPORT LazyLoadSheetIFace : public MSheet
{
public:
  virtual void createContent() = 0;
  virtual void destroyContent() = 0;
  virtual bool isContentCreated() const = 0;
};

/**
 * \brief A templated page class that creates a widget of type T
 * and sets it as the central widget when the sheet is made visible.
 */
template <class T> class LazyLoadSheet : public LazyLoadSheetIFace
{
public:
  /**
   * \brief Constructor.
   */
  LazyLoadSheet()
    :m_widget(0)
  {
  }

  /**
   * \brief Destructor.
   */
  ~LazyLoadSheet()
  {
  }

  /**
   * \brief Return a handle to the widget.
   */
  T* widget()
  {
    Q_ASSERT(m_widget);
    return m_widget;
  }

  /**
   * \brief Get page viewport
   *
   * Note: Should only  be called after page content has been created.
   */
  MPannableViewport* viewPort() const
  {
      return m_viewport;
  }

  /**
   * \brief Called by the PageSwitcher to create page content before
   * displaying the page.
   */
  virtual void createContent()
  {
    if (m_widget)
      return;


    m_viewport = new MPannableViewport(this);
    m_viewport->setClipping(true);
    m_widget = new T(m_viewport);
    m_viewport->setWidget(m_widget);
    m_viewport->positionIndicator()->setEnabled(false);

    setCentralWidget(m_viewport);

    MBasicSheetHeader *basicHeader = new MBasicSheetHeader(this);
    setHeaderWidget(basicHeader);
  }

  bool isContentCreated() const
  {
      return (m_widget != 0);
  }

  /**
   * \brief Destroys the content.
   */
  virtual void destroyContent()
  {
    if (!m_widget)
      return;

    // The widget must be deleted by calling setCentralWidget or we crash
    // when setCentralWidget is called on content recreation.
    setCentralWidget(0);
    m_widget = 0;
  }

private:
  T* m_widget;
  QGraphicsLinearLayout *m_layout;
  MPannableViewport* m_viewport;
};

/**
 * \brief Implement this interface in a widget class used with PageSwitcher
 * to receive notifications about page switches concerning that widget.
 */
class ORGANISER_EXPORT PageSwitchObserverIface
{
public:
  /**
   * \brief Called when about to switch away from a page due to a back button
   * click. If the return value is false, the back click is ignored.
   */
  virtual bool backClicked()
  {
    return true;
  }

};

/**
 * \brief A class for managing page switching.
 */
class ORGANISER_EXPORT PageSwitcher : public QObject
{
  Q_OBJECT

public:
  /**
   * \brief Get a singleton instance of the class.
   */
  static PageSwitcher& instance()
  {
    static PageSwitcher singleton;
    return singleton;
  }

  /**
   * \brief Show the specified page.
   *
   * The page can be but doesn't have to be a LazyLoadPage.
   *
   * \param page The page to insert.
   * \param replace If true, replace the entire contents of the page stack
   * with this page.
   * \param skipAnimation If true, skip transition animation. Replace implies
   * skipAnimation.
   *
   */
  void showPage(MApplicationPage* page, bool replace=false, bool skipAnimation=false);

  /**
   * \brief Show the specified page.
   *
   * The page can be but doesn't have to be a LazyLoadPage.
   *
   * \param page The page to insert.
   * \param replace If true, replace the entire contents of the page stack
   * with this page.
   * \param skipAnimation If true, skip transition animation. Replace implies
   * skipAnimation.
   *
   */
  void showPage(MSheet* page, bool replace=false, bool skipAnimation=false);

  /**
   * \brief Show the currently showing page again.
   *
   * Call this function if you have shown a page yourself and want to show
   * the latest page managed by PageSwitcher again.
   *
   * \param skipAnimation If true, skip transition animation.
   */
  void showLatestPage(bool skipAnimation=false);

  /**
   * \brief Get the page that is currently showing.
   *
   * \return The top page.
   */
  MApplicationPage* latestPage();

  /**
   * \brief Get the window that is currently showing.
   *
   * \return The top window.
   */
  MSceneWindow* latestWindow() const;

  /**
   * \brief Get the nth window from the top.
   *
   * \param n Number of the page to get.
   * \return The page or 0 if no such page.
   */
  MSceneWindow* pageFromTop(int n);

  /**
   * \brief Pop N pages from the stack.
   *
   * That is, remove last N pages from stack.
   *
   * \param skipAnimation If true, skip transition animation.
   */
  void popPages(int count, bool skipAnimation=false);

  /**
   * \brief Registers the dialog for auto killing.
   *
   * Registered dialogs are killed when killDialogs is called by calling their
   * reject method. The dialogs are automatically unregistered when they
   * exit.
   */
  void registerAutokillDialog(MDialog* dialog);

  /**
   * \brief Unregisters the dialog.
   */
  void unregisterAutokillDialog(MDialog* dialog);

  /**
   * \brief Kills all dialogs registered for auto killing.
   *
   * This does not include dialogs that were registered but have exited
   * since. They are automatically unregistered.
   */
  void killDialogs();

  /**
   * \brief Forces animation on next page switch by inserting the new page
   * into libmeegotouch's page stack.
   */
  void forceAnimationNextTime();

private:
  Q_DISABLE_COPY(PageSwitcher);
  Q_DECLARE_PRIVATE(PageSwitcher);
  PageSwitcherPrivate* d_ptr;

  PageSwitcher();
  ~PageSwitcher();

public slots:
  /**
   * \brief Call this slot manually to simulate a back button click.
   */
  void backClicked();

  void hideTopSheetPage();
  void hideTopSheetPageNow();

private slots:
  void dialogDestroyed(QObject* dialog);

};

#endif
